#pragma once


#include <ArmarXCore/core/ComponentPlugin.h>
#include <ArmarXCore/core/ManagedIceObject.h>

#include <armarx/navigation/client/Navigator.h>
#include <armarx/navigation/client/ice/NavigatorInterface.h>
#include <armarx/navigation/client/services/IceNavigator.h>
#include <armarx/navigation/client/services/MemorySubscriber.h>
#include <armarx/navigation/client/services/SimpleEventHandler.h>


namespace armarx::navigation::client
{

    class ComponentPlugin : virtual public armarx::ComponentPlugin
    {

    public:
        using armarx::ComponentPlugin::ComponentPlugin;

        ~ComponentPlugin() override;

        void postCreatePropertyDefinitions(armarx::PropertyDefinitionsPtr& properties) override;

        void preOnInitComponent() override;

        void preOnConnectComponent() override;

    private:
        static constexpr const char* PROPERTY_NAME = "nav.NavigatorName";

        client::NavigatorInterfacePrx navigatorPrx;
    };


    class ComponentPluginUser : virtual public ManagedIceObject
    {

    public:
        ComponentPluginUser();

        void configureNavigator(const client::NavigationStackConfig& stackConfig);

        void configureNavigator(const client::NavigationStackConfig& stackConfig,
                                const std::string& configId);


        // Non-API

    public:
        ~ComponentPluginUser() override;

        friend class ComponentPlugin;

    private:
        IceNavigator iceNavigator;
        // MemorySubscriber memorySubscriber;
        SimpleEventHandler memorySubscriber; // FIXME
        Navigator::InjectedServices navigatorServices{.navigator = &iceNavigator,
                                                      .subscriber = &memorySubscriber};


        ComponentPlugin* plugin = nullptr;


    public:
        Navigator navigator;
    };

} // namespace armarx::navigation::client
