#include "GlobalPlannerFactory.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>
#include <RobotAPI/libraries/aron/core/data/variant/primitive/String.h>

#include <armarx/navigation/core/constants.h>
#include <armarx/navigation/global_planning/AStar.h>
#include <armarx/navigation/global_planning/Point2Point.h>
#include <armarx/navigation/global_planning/core.h>

namespace armarx::navigation::fac
{
    glob_plan::GlobalPlannerPtr
    GlobalPlannerFactory::create(const aron::data::DictPtr& params,
                                 const core::Scene& ctx)
    {
        namespace layer = glob_plan;

        if (not params)
        {
            return nullptr;
        }

        // algo name
        const auto algoName =
            aron::data::String::DynamicCast(params->getElement(core::NAME_KEY));
        ARMARX_CHECK_NOT_NULL(algoName);
        const layer::Algorithms algo = layer::AlgorithmNames.from_name(algoName->getValue());

        // algo params
        const auto algoParams =
            aron::data::Dict::DynamicCast(params->getElement(core::PARAMS_KEY));
        ARMARX_CHECK_NOT_NULL(algoParams);

        glob_plan::GlobalPlannerPtr globalPlanner;
        switch (algo)
        {
            case glob_plan::Algorithms::AStar:
                globalPlanner = std::make_shared<glob_plan::AStar>(
                    glob_plan::AStarParams::FromAron(algoParams), ctx);
                break;
            case glob_plan::Algorithms::Point2Point:
                globalPlanner = std::make_shared<glob_plan::Point2Point>(
                    glob_plan::Point2PointParams::FromAron(algoParams), ctx);
                break;
        }

        return globalPlanner;
    }

} // namespace armarx::navigation::fac
